
"""
<name>Rank selection for NMF</name>
<description>Compute measures for NMF rank selection</description>
<icon>icons/NMF.png</icon>
<priority>30</priority>
    """


import nimfa
import numpy as np

#import sys
#sys.path.append("C:\\Python27\\Lib\site-packages\\Orange\\OrangeWidgets")
from OWWidget import *
import OWGUI



class OWNMFRankSelection(OWWidget):
    settingsList = ['nbIter','factorizationMethod','rangeRanksUB','rangeRanksLB','initialization','nbRuns']
    def __init__(self, parent=None, signalManager=None):
        OWWidget.__init__(self, parent, signalManager, 'Rank selection for NMF')
        
        self.inputs = [("Data", ExampleTable, self.data),("Initial_W", ExampleTable, self.dataInitW),("Initial_H", ExampleTable, self.dataInitH)]
        self.outputs = [("Measures", ExampleTable)]
        
        self.nbIter=100
        self.factorizationMethod=0
        self.initialization=0
        self.rangeRanksUB=10 # upper bound for range of ranks
        self.rangeRanksLB=2 # lower bound for range of ranks
        self.nbRuns=10 # number of runs to calculate measures
        self.doCoph=1
        self.doSparse=0
        self.doRss=0
        self.loadSettings()
        
        # GUI
        box = OWGUI.widgetBox(self.controlArea, "Info")
        self.infoa = OWGUI.widgetLabel(box, 'No data on input yet, waiting to get something.')
        self.infob = OWGUI.widgetLabel(box, '')
        
        OWGUI.separator(self.controlArea)
        self.paramBox = OWGUI.widgetBox(self.controlArea, "Parameters")
        OWGUI.lineEdit(self.paramBox, self, 'nbIter', label='Number of iterations',labelWidth=150, orientation="horizontal")
        self.methodBox = OWGUI.widgetBox(self.controlArea, "Factorization Method")        
        OWGUI.radioButtonsInBox(self.methodBox, self, 'factorizationMethod',callback=self.setNbIter,
                                btnLabels = ["NMF", "LSNMF", "SNMF", "BMF"],
                                tooltips = ["Standard Nonnegative Matrix Factorization",
                                            "Alternating Nonnegative Least Squares Matrix Factorization Using Projected Gradient (bound constrained optimization) method for each subproblem",
                                            "Sparse Nonnegative Matrix Factorization",
                                            "Binary Matrix Factorization"])
        self.initBox = OWGUI.widgetBox(self.controlArea, "Initialization")        
        OWGUI.radioButtonsInBox(self.initBox, self, 'initialization',
                                btnLabels = ["random_vcol", "NNDSVD", "Fixed"],
                                tooltips = ["Random initialization of factors",
                                            "Nonnegative Double Singular Value Decomposition",
                                            "Fixed (to be done)"])
        self.chooseRankBox = OWGUI.widgetBox(self.controlArea, "Find best rank (calculate selected measures)")
        self.infob = OWGUI.widgetLabel(box, '')
        OWGUI.spin(self.chooseRankBox, self, 'rangeRanksLB', label='Minimum rank',min=1,max=200, step=1,labelWidth=150, orientation="horizontal")
        OWGUI.spin(self.chooseRankBox, self, 'rangeRanksUB', label='Maximum rank',min=1,max=200, step=1,labelWidth=150, orientation="horizontal")   
        OWGUI.spin(self.chooseRankBox, self, 'nbRuns', label='Number of runs',min=1,max=100, step=1,labelWidth=150, orientation="horizontal")
        OWGUI.checkBox(self.chooseRankBox, self, "doCoph", "Cophenetic correlation coefficient", callback=self.setNbRuns)
        OWGUI.checkBox(self.chooseRankBox, self, "doRss", "RSS", callback=self.setNbRuns)        
        OWGUI.checkBox(self.chooseRankBox, self, "doSparse", "Sparseness", callback=self.setNbRuns)
        OWGUI.button(self.controlArea, self, "Go (this process may be lengthy)", callback=self.findBestRank)        
        self.paramBox.setDisabled(1)
        self.methodBox.setDisabled(1)
        self.initBox.setDisabled(1)
        self.chooseRankBox.setDisabled(1)
        
        self.resize(100,50)
    
    def data(self, dataset):
        if dataset:
            self.dataset = dataset
            self.infoa.setText('%d variables in input data set' % len(dataset[0]))
            self.infob.setText('%d observations in input data set' % len(dataset))
            self.paramBox.setDisabled(0)
            self.methodBox.setDisabled(0)
            self.initBox.setDisabled(0)
            self.chooseRankBox.setDisabled(0)
        #self.commit()
        else:
            self.send("W", None)
            self.send("H", None)
            self.send("Measures",None)
            self.infoa.setText('No data on input yet, waiting to get something.')
            self.infob.setText('')   
    
    def dataInitH(self, dataset):
        if not dataset:
            return # avoid any unnecessary computation
        H=dataset.toNumpy()
        H=np.matrix(H[0]) 
        self.initH = H
    
    def dataInitW(self, dataset):
        if not dataset:
            return # avoid any unnecessary computation
        W=dataset.toNumpy()
        W=np.matrix(W[0]) 
        self.initW = W
        
    def setNbIter(self):
        if self.factorizationMethod==0:
            self.nbIter=200
        elif self.factorizationMethod==1:
            self.nbIter=200
        elif self.factorizationMethod==2:
            self.nbIter=200
        else:
            self.nbIter=200 
    
    def setNbRuns(self):
        if self.doCoph:
            self.nbRuns=10
        else:
            self.nbRuns=1
    
    def findBestRank(self):   
        V=self.dataset.toNumpy()
        V=np.matrix(V[0])
        methodLabels = ["nmf", "lsnmf", "snmf", "bmf"]
        initLabels = ["random_vcol", "nndsvd", "fixed"]
        if self.initialization == 2:
            fctr = nimfa.mf(V, seed = initLabels[self.initialization], method = methodLabels[self.factorizationMethod], rank = 4, max_iter = self.nbIter, W=self.initW, H=self.initH)
        else:
            if self.factorizationMethod==1:
                fctr = nimfa.mf(V, seed = initLabels[self.initialization], min_residuals=1e-15, method = methodLabels[self.factorizationMethod], rank = 4, max_iter = self.nbIter, sub_iter = 20, inner_sub_iter = 20)
            else:
                fctr = nimfa.mf(V, seed = initLabels[self.initialization], method = methodLabels[self.factorizationMethod], rank = 4, max_iter = self.nbIter) 
        measures=[]
        if self.doCoph:
            measures.append('cophenetic')
        if self.doRss:
            measures.append('rss')
        if self.doSparse:
            measures.append('sparseness')
        res = fctr.estimate_rank(range=xrange(self.rangeRanksLB,self.rangeRanksUB +1),n_run=self.nbRuns, what=measures)
        if self.doSparse:
            resTable=np.zeros([self.rangeRanksUB-self.rangeRanksLB+1,len(measures)+2])
        else:
            resTable=np.zeros([self.rangeRanksUB-self.rangeRanksLB+1,len(measures)+1]) 
        # fill output table, since sparseness is computed for both W and H, treat it separately (two columns instead of one)    
        for i in range(0, self.rangeRanksUB-self.rangeRanksLB+1):
            resTable[i,0]=self.rangeRanksLB + i
            if self.doCoph:
                resTable[i, 1]= res[resTable[i,0]]['cophenetic']
            if self.doRss:
                resTable[i, self.doCoph + 1]= res[resTable[i,0]]['rss']
            if self.doSparse:
                resTable[i,len(measures)]=res[resTable[i,0]]['sparseness'][0]
                resTable[i,len(measures)+1]=res[resTable[i,0]]['sparseness'][1]
        
        # transform to Orange format        
        domResList=[Orange.feature.Continuous("Rank")]
        if self.doCoph:
            domResList.append(Orange.feature.Continuous("Cophenetic"))
        if self.doRss:
            domResList.append(Orange.feature.Continuous("RSS"))
        if self.doSparse:
            domResList.append(Orange.feature.Continuous("Sparseness W"))
            domResList.append(Orange.feature.Continuous("Sparseness H"))
        self.listAtt=domResList
        domRes=Orange.data.Domain(domResList,0)
        self.measures=Orange.data.Table(domRes,resTable)        
        self.send("Measures", self.measures)



# Test the widget, run from prompt

if __name__=="__main__":
    appl = QApplication(sys.argv)
    ow = OWNMFRankSelection()
    ow.show()
    dataset = orange.ExampleTable('C:/Users/Fajwel/Dropbox/Orange/sampleData.tab')
    ow.data(dataset)
    ow.dataInitH(dataset)
    ow.dataInitW(dataset)
    appl.exec_()





"""
<name>NMF</name>
<description>Non negative matrix factorization</description>
<icon>icons/NMF.png</icon>
<priority>30</priority>
"""

import nimfa
import numpy as np
import scipy.stats


#import sys
#sys.path.append("C:\\Python27\\Lib\site-packages\\Orange\\OrangeWidgets")
from OWWidget import *
import OWGUI
import re, os.path
from exceptions import Exception


class OWNMF(OWWidget):
    settingsList = ['rank','nbIter','factorizationMethod','initialization','displayCluster','reorderingOption','saveOutputs',"recentFiles","selectedFileName"]
    # things for saving:    
    savers = {".txt": orange.saveTxt, ".tab": orange.saveTabDelimited,
              ".names": orange.saveC45, ".test": orange.saveC45, ".data": orange.saveC45,
              ".csv": orange.saveCsv
              }

    # exclude C50 since it has the same extension and we do not need saving to it anyway
    registeredFileTypes = [ft for ft in orange.getRegisteredFileTypes() if len(ft)>3 and ft[3] and not ft[0]=="C50"]

    dlgFormats = 'Tab-delimited files (*.tab)\nHeaderless tab-delimited (*.txt)\nComma separated (*.csv)\nC4.5 files (*.data)\nRetis files (*.rda *.rdo)\n' \
                 + "\n".join("%s (%s)" % (ft[:2]) for ft in registeredFileTypes) \
                 + "\nAll files(*.*)"

    savers.update(dict((lower(ft[1][1:]), ft[3]) for ft in registeredFileTypes))
    re_filterExtension = re.compile(r"\(\*(?P<ext>\.[^ )]+)")
    
    def __init__(self, parent=None, signalManager=None):
        OWWidget.__init__(self, parent, signalManager, 'NMFV7')
        
        self.inputs = [("Data", ExampleTable, self.dataX),("Xd", ExampleTable, self.dataXd),("Xa", ExampleTable, self.dataXa),("Initial_W", ExampleTable, self.dataInitW),("Initial_H", ExampleTable, self.dataInitH)]
        self.outputs = [("W", ExampleTable), ("H", ExampleTable),("S", ExampleTable),("WSH",ExampleTable),("Residuals",ExampleTable),("qqplot",ExampleTable),("Reordered X",ExampleTable),("Reordered WSH",ExampleTable),("Reordered residuals",ExampleTable)]

        self.rank = 4
        self.nbIter=200
        self.factorizationMethod=0
        self.initialization=0
        self.displayCluster=0
        self.reorderingOption=2
        self.saveOutputs=1
        
        # things for saving
        self.recentFiles=[]
        self.selectedFileName = "None"
        self.data = None
        self.filename = ""  
        
        self.loadSettings()

        # GUI
        box = OWGUI.widgetBox(self.controlArea, "Info")
        self.infoa = OWGUI.widgetLabel(box, 'No data on input yet, waiting to get something.')
        self.infob = OWGUI.widgetLabel(box, '')

        OWGUI.separator(self.controlArea)
        self.paramBox = OWGUI.widgetBox(self.controlArea, "Parameters")
        OWGUI.spin(self.paramBox, self, 'rank', min=1, max=200, step=1,
                   label='Rank',labelWidth=150)
        OWGUI.lineEdit(self.paramBox, self, 'nbIter', label='Number of iterations',labelWidth=150, orientation="horizontal")
        self.methodBox = OWGUI.widgetBox(self.controlArea, "Factorization Method")        
        OWGUI.radioButtonsInBox(self.methodBox, self, 'factorizationMethod', callback=self.setNbIter,
              btnLabels = ["NMF", "LSNMF", "SNMF", "BMF"],
              tooltips = ["Standard Nonnegative Matrix Factorization",
                          "Alternating Nonnegative Least Squares Matrix Factorization Using Projected Gradient (bound constrained optimization) method for each subproblem",
                          "Sparse Nonnegative Matrix Factorization",
                          "Binary Matrix Factorization"])
        self.initBox = OWGUI.widgetBox(self.controlArea, "Initialization")        
        OWGUI.radioButtonsInBox(self.initBox, self, 'initialization',
              btnLabels = ["random_vcol", "NNDSVD", "Fixed"],
              tooltips = ["Random initialization of factors",
                          "Nonnegative Double Singular Value Decomposition",
                          "Fixed (to be done)"])
        self.visualizationBox=OWGUI.widgetBox(self.controlArea, "Visualization options")
        OWGUI.checkBox(self.visualizationBox, self, "displayCluster", "Display cluster in output data")
        self.reorderingBox=OWGUI.widgetBox(self.visualizationBox, "Reorder")
        OWGUI.radioButtonsInBox(self.reorderingBox, self, 'reorderingOption',
              btnLabels = ["Rows", "Columns", "Both rows and columns"],
              tooltips = ["Reorder rows according to W", "Reorder columns according to H", "Reorder both rows and columns"])
        OWGUI.button(self.controlArea, self, "Commit", callback=self.commit)
        self.saveBox = OWGUI.widgetBox(self.controlArea, "Saving options")
        OWGUI.checkBox(self.saveBox, self, "saveOutputs", "Save outputs")
        rfbox = OWGUI.widgetBox(self.saveBox, "Filename", orientation="horizontal", addSpace=True)
        self.filecombo = OWGUI.comboBox(rfbox, self, "filename")
        self.filecombo.setMinimumWidth(200)
        button = OWGUI.button(rfbox, self, '...', callback = self.browseFile, disabled=0)
        button.setIcon(self.style().standardIcon(QStyle.SP_DirOpenIcon))
        button.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Fixed)
        self.resend = OWGUI.button(self.saveBox, self, "Retreive data", callback = self.resendData, default=True)
        
        OWGUI.rubber(self.controlArea)
        self.setFilelist()
        #self.resize(260,100)
        self.filecombo.setCurrentIndex(0)

        if self.selectedFileName != "":
            if os.path.exists(self.selectedFileName):
                self.openFile(self.selectedFileName)
            else:
                self.selectedFileName = ""      
        
        self.infob = OWGUI.widgetLabel(box, '')        
        self.paramBox.setDisabled(1)
        self.methodBox.setDisabled(1)
        self.initBox.setDisabled(1)
        self.visualizationBox.setDisabled(1)
        self.saveBox.setDisabled(1)
        
        self.resize(100,50)        
        
    def dataX(self, dataset):
        if dataset:
            self.dataset = dataset
            self.infoa.setText('%d variables in input data set' % len(dataset[0]))
            self.infob.setText('%d observations in input data set' % len(dataset))
            self.paramBox.setDisabled(0)
            self.methodBox.setDisabled(0)
            self.initBox.setDisabled(0)
            self.visualizationBox.setDisabled(0)
            self.saveBox.setDisabled(0)
        else:
            self.send("W", None)
            self.send("H", None)
            self.send("WSH",None)
            self.send("S", None)
            self.send("Residuals",None)
            self.send("qqplot",None)
            self.send("Reordered X", None)
            self.send("Reordered WSH", None)
            self.send("Reordered residuals", None)
            self.infoa.setText('No data on input yet, waiting to get something.')
            self.infob.setText('')
            

    def dataXd(self, dataset):
        if not dataset:
            self.Xd=None
        else:
            self.Xd = dataset   
    
    def dataXa(self, dataset):
        if not dataset:
            self.Xa=None
        else:
            self.Xa= dataset 
    
    def dataInitH(self, dataset):
        if not dataset:
            return # avoid any unnecessary computation
        H=dataset.toNumpy()
        H=np.matrix(H[0]) 
        self.initH = H.transpose

    def dataInitW(self, dataset):
        if not dataset:
            return # avoid any unnecessary computation
        W=dataset.toNumpy()
        W=np.matrix(W[0]) 
        self.initW = W
    
    def setNbIter(self):
        if self.factorizationMethod==0:
            self.nbIter=200
        elif self.factorizationMethod==1:
            self.nbIter=200
        elif self.factorizationMethod==2:
            self.nbIter=200
        else:
            self.nbIter=200 

    def runNMF(self):
        V=self.dataset.toNumpy()
        V=np.matrix(V[0])
        methodLabels = ["nmf", "lsnmf", "snmf", "bmf"]
        initLabels = ["random_vcol", "nndsvd", "fixed"]
        if self.initialization == 2:
            fctr = nimfa.mf(V, seed = initLabels[self.initialization], method = methodLabels[self.factorizationMethod], rank = self.rank, max_iter = self.nbIter, W=self.initW, H=self.initH)
        else:
            if self.factorizationMethod==1:
                fctr = nimfa.mf(V, seed = initLabels[self.initialization], min_residuals=1e-15, method = methodLabels[self.factorizationMethod], rank = self.rank, max_iter = self.nbIter, sub_iter = 20, inner_sub_iter = 20)
            else:
                fctr = nimfa.mf(V, seed = initLabels[self.initialization], method = methodLabels[self.factorizationMethod], rank = self.rank, max_iter = self.nbIter)
        fctr_res = nimfa.mf_run(fctr)
        W=fctr_res.basis()
        H=fctr_res.coef()
        WH=W*H
        residuals=V-WH
        # renorm W and H and deduce S (factorization is then WSH)
        self.Wnorm=np.array(W)
        self.Hnorm=np.array(H)
        self.S=np.zeros([self.rank,1]) # scaling vector        
        nbComp=self.Wnorm.shape[1]           
        for j in range(0,nbComp):
            sumw=np.sqrt(sum(self.Wnorm[:,j]**2))
            sumh=np.sqrt(sum(self.Hnorm[j,:]**2))
            self.Wnorm[:,j]=self.Wnorm[:,j]/sumw
            self.Hnorm[j,:]=self.Hnorm[j,:]/sumh
            self.S[j,0]=sumw*sumh
        
        self.residualsNP=residuals #save residuals in numpy format
        self.WHNP=np.array(WH) #save WSH in numpy format       
        
        if self.displayCluster: # last column of W, WSH and residuals is the cluster assigned to the data (highest weight component index)   
            self.clusterCol=np.matrix(self.Wnorm).argmax(1)+1
            self.clusterCol=np.array(self.clusterCol)
        domainX=self.dataset.domain
     # Transform to Orange format   
        # Transpose H
        self.HNP=np.transpose(self.Hnorm)
        self.HH=Orange.data.Table(self.HNP)        
        # concatenate with Xa if available
        if hasattr(self, 'Xa') and self.Xa:
            self.HH=Orange.data.Table([self.HH,self.Xa])
        else:
            self.HH.domain.add_meta(Orange.feature.Descriptor.new_meta_id(), Orange.feature.String("Variable"))
            # Get the names of the variables as meta attribute
            for i in range(0,len(domainX)):
                self.HH[i]["Variable"]=domainX[i].name
        
        newDomain=domainX
        domW=[]
        for i in range(0, self.rank):
            domW.append(Orange.feature.Continuous("Component %i" %(i+1)))
        domW=Orange.data.Domain(domW,0)
        domW.addmetas(domainX.getmetas())
        self.WW = Orange.data.Table(domW,self.Wnorm)
        self.WSH = Orange.data.Table(newDomain,np.array(WH))
        self.S=Orange.data.Table(Orange.data.Domain([Orange.feature.Continuous("Scaling factor")],0),self.S)
        self.residuals= Orange.data.Table(newDomain,np.array(residuals))
        if self.displayCluster:
            self.clusterCol=self.clusterCol.transpose()[0].tolist()
            for i in range(0, len(self.clusterCol)):
                self.clusterCol[i]=[str(self.clusterCol[i])]
            val=[]
            for i in range(1,self.rank+1):
                val.append(str(i))
            domCluster=Orange.data.Domain([Orange.feature.Discrete("Cluster",values=val)])
            clusterTable=Orange.data.Table(domCluster,self.clusterCol)
            #concatenate W, WSH and residuals with cluster table
            self.WW=Orange.data.Table([self.WW,clusterTable])
            self.WSH=Orange.data.Table([self.WSH,clusterTable])
            self.residuals=Orange.data.Table([self.residuals,clusterTable])
                           
        if hasattr(self, 'Xd') and self.Xd:
            self.WW =  Orange.data.Table([self.WW,self.Xd]) 
            self.WSH = Orange.data.Table([self.WSH,self.Xd])
            self.residuals = Orange.data.Table([self.residuals,self.Xd])
        else:
            for i in range(0,len(self.dataset)):
                self.WW[i]["ID"]=self.dataset[i]["ID"]
                self.WSH[i]["ID"]=self.dataset[i]["ID"]
                self.residuals[i]["ID"]=self.dataset[i]["ID"]
                
            
    def calculateQQPlotValues(self):
        nbValues=self.residualsNP.shape[0]*self.residualsNP.shape[1]
        # put all residuals in one vector        
        residualsVector=np.array(np.reshape(self.residualsNP, nbValues))[0]
        # sort residuals
        residualsVector.sort()
        normalQuantilesVector=scipy.stats.norm.ppf(np.linspace(1./nbValues,1-1./nbValues,nbValues),loc=residualsVector.mean(),scale=residualsVector.std())
        self.qqplot=np.array([normalQuantilesVector,residualsVector])
        self.qqplot=self.qqplot.transpose()
        domQQPlot=Orange.data.Domain([Orange.feature.Continuous("normal quantiles"),Orange.feature.Continuous("residuals quantiles")],0)
        self.qqplot=Orange.data.Table(domQQPlot,self.qqplot)
        
    def reorder(self):
        if self.reorderingOption==0:
            orderLines=1
            orderCol=0
        else:
            if self.reorderingOption==1:
                orderLines=0
                orderCol=1
            else:
                orderLines=1
                orderCol=1
                
        self.Xordered=self.dataset.toNumpy()[0]
        self.WSHordered=self.WHNP
        self.residualsOrdered=np.array(self.residualsNP)
        varOrdered=Orange.data.Domain(self.dataset.domain)
        IDordered=self.dataset
        if hasattr(self, 'Xd') and self.Xd:
            Xdordered=self.Xd
        if orderLines:
            # normalize weights
            Wnorm=self.Wnorm
            nbComp=self.Wnorm.shape[1]  
            # Assign each observation to a cluster corresponding to its largest weight
            cluster=Wnorm.argmax(1)       
            
            # Get indices of observation in each cluster
            ind=[]
            for j in range(0,nbComp):
                ind.append(np.nonzero(cluster==j)[0])
            # reorder each cluster by descending sorting of weight
            orderLin=[]            
            for j in range(0,nbComp):
                indComp=ind[j]                
                orderLin.append(indComp[Wnorm[indComp,j].argsort()[::-1]])
            # concatenate the orderCol in one array
            finalOrderLin=orderLin[0]            
            for j in range(1,nbComp):
                finalOrderLin=np.concatenate((finalOrderLin,orderLin[j]),axis=0)
            
            self.Xordered=self.Xordered[finalOrderLin,:]
            self.WSHordered=self.WSHordered[finalOrderLin,:]
            self.residualsOrdered=self.residualsOrdered[finalOrderLin,:]
            
            if self.displayCluster: # last column of W, WSH and residuals is the cluster assigned to the data (highest weight component index)   
                self.clusterCol=np.matrix(Wnorm).argmax(1) + 1
                self.clusterCol=self.clusterCol[finalOrderLin,:]
                self.clusterCol=np.array(self.clusterCol).transpose()[0].tolist()
                for i in range(0, len(self.clusterCol)):
                    self.clusterCol[i]=[str(self.clusterCol[i])]
                    
            # reorder IDs
            finalOrderLin=finalOrderLin.tolist()            
            IDordered=Orange.data.Table([self.dataset[finalOrderLin[0]]])
            for i in range(1,len(finalOrderLin)):
                IDordered.append(self.dataset[finalOrderLin[i]])
            # reorder Xd:                 
            if hasattr(self, 'Xd') and self.Xd:               
                Xdordered=Orange.data.Table([self.Xd[finalOrderLin[0]]])
                for i in range(1,len(finalOrderLin)):
                    Xdordered.append(self.Xd[finalOrderLin[i]])
                Xdordered=Orange.data.Table(self.Xd.domain,Xdordered)
                
        if orderCol: #same as with ordering of lines but work with H instead or W and columns instead of lines
            # normalize H (which is already transposed)
            Hnorm=self.Hnorm.T
            nbComp=Hnorm.shape[1]           
            # Assign each variable to a cluster corresponding to its largest H
            cluster=Hnorm.argmax(1)
            # Get indices of variable in each cluster
            ind=[]
            for j in range(0,nbComp):
                ind.append(np.nonzero(cluster==j)[0])
            # reorder each cluster by descending sorting of H
            orderCol=[]            
            for j in range(0,nbComp):
                indComp=ind[j]                
                orderCol.append(indComp[Hnorm[indComp,j].argsort()[::-1]])
            # concatenate the orderCol in one array
            finalOrderCol=orderCol[0]            
            for j in range(1,nbComp):
                finalOrderCol=np.concatenate((finalOrderCol,orderCol[j]),axis=0)
            
            self.Xordered=self.Xordered[:,finalOrderCol]
            self.WSHordered=self.WSHordered[:,finalOrderCol]
            self.residualsOrdered=self.residualsOrdered[:,finalOrderCol]            
            
            # reorder Variables
            finalOrderCol=finalOrderCol.tolist()            
            varOrdered=[self.dataset.domain[finalOrderCol[0]]]
            for i in range(1,len(self.dataset.domain)):
                varOrdered.append(self.dataset.domain[finalOrderCol[i]])
            varOrdered=Orange.data.Domain(varOrdered,0)
            varOrdered.addmetas(self.dataset.domain.getmetas())#add "ID" meta attribute 
 
        if self.displayCluster:  
            val=[]
            for i in range(1,self.rank+1):
                val.append(str(i))
            domCluster=Orange.data.Domain([Orange.feature.Discrete("Cluster",values=val)])            
            clusterTable=Orange.data.Table(domCluster,self.clusterCol)
        
        # return Orange table
        self.Xordered = Orange.data.Table(varOrdered,np.array(self.Xordered))
        self.WSHordered = Orange.data.Table(varOrdered,np.array(self.WSHordered))
        self.residualsOrdered = Orange.data.Table(varOrdered,np.array(self.residualsOrdered))
        if self.displayCluster:
            #concatenate Xordered with cluster table
            self.Xordered=Orange.data.Table([self.Xordered,clusterTable])
            self.WSHordered=Orange.data.Table([self.WSHordered,clusterTable])
            self.residualsOrdered=Orange.data.Table([self.residualsOrdered,clusterTable])
        if hasattr(self, 'Xd') and self.Xd:
            self.Xordered =  Orange.data.Table([self.Xordered,Xdordered])
            self.WSHordered =  Orange.data.Table([self.WSHordered,Xdordered])
            self.residualsOrdered =  Orange.data.Table([self.residualsOrdered,Xdordered])
        else:                    
            for i in range(0,len(self.dataset)):
                self.Xordered[i]["ID"]=IDordered[i]["ID"]
                self.WSHordered[i]["ID"]=IDordered[i]["ID"]
                self.residualsOrdered[i]["ID"]=IDordered[i]["ID"]
    
    def browseFile(self):
        if self.recentFiles:
            startfile = self.recentFiles[0]
        else:
            startfile = os.path.expanduser("~/")

#        filename, selectedFilter = QFileDialog.getSaveFileNameAndFilter(self, 'Save Orange Data File', startfile,
#                        self.dlgFormats, self.dlgFormats.splitlines()[0])
#        filename = str(filename)
#       The preceding lines should work as per API, but do not; it's probably a PyQt bug as per March 2010.
#       The following is a workaround.
#       (As a consequence, filter selection is not taken into account when appending a default extension.)
        filename, selectedFilter = QFileDialog.getSaveFileName(self, 'Save Orange Data File', startfile,
                         self.dlgFormats), self.dlgFormats.splitlines()[0]
        filename = unicode(filename)
        if not filename or not os.path.split(filename)[1]:
            return

        ext = lower(os.path.splitext(filename)[1])
        if not ext in self.savers:
            filt_ext = self.re_filterExtension.search(str(selectedFilter)).group("ext")
            if filt_ext == ".*":
                filt_ext = ".tab"
            filename += filt_ext

        self.addFileToList(filename)
        if hasattr(self, 'WW') and self.WW:      
            self.saveFile()

    def saveFile(self, *index):
        self.error()
        #if self.data is not None:
        combotext = unicode(self.filecombo.currentText())
        if combotext == "(none)":
            QMessageBox.information( None, "Error saving data", "Unable to save data. Select first a file name by clicking the '...' button.", QMessageBox.Ok + QMessageBox.Default)
            return
        filename = self.recentFiles[self.filecombo.currentIndex()]
        fileExt = lower(os.path.splitext(filename)[1])
        filenameWithoutExt=lower(os.path.splitext(filename)[0])
        WfileName=filenameWithoutExt + "_W" + ".tab"
        HfileName=filenameWithoutExt + "_H" + ".tab"
        SfileName=filenameWithoutExt + "_S" + ".tab"
        ResidualsfileName=filenameWithoutExt + "_Residuals" + ".tab"
        WSHfileName=filenameWithoutExt + "_WSH" + ".tab"
        QQplotfileName=filenameWithoutExt + "_QQplot" + ".tab"
        ReorderedXfileName=filenameWithoutExt + "_Reordered_X" + ".tab"
        ReorderedWSHfileName=filenameWithoutExt + "_Reordered_WSH" + ".tab"
        ReorderedResidualsfileName=filenameWithoutExt + "_Reordered_residuals" + ".tab"
        if fileExt == "":
            fileExt = ".tab"
        try:
            self.savers[fileExt](WfileName, self.WW)
            self.savers[fileExt](HfileName, self.HH)
            self.savers[fileExt](SfileName, self.S)
            self.savers[fileExt](ResidualsfileName, self.residuals)
            self.savers[fileExt](WSHfileName, self.WSH)
            self.savers[fileExt](QQplotfileName, self.qqplot)
            self.savers[fileExt](ReorderedXfileName, self.Xordered)
            self.savers[fileExt](ReorderedWSHfileName, self.WSHordered)
            self.savers[fileExt](ReorderedResidualsfileName, self.residualsOrdered)
        except Exception, (errValue):
            self.error(str(errValue))
            return
        self.error()

    def addFileToList(self,fn):
        if fn in self.recentFiles:
            self.recentFiles.remove(fn)
        self.recentFiles.insert(0,fn)
        self.setFilelist()

    def setFilelist(self):
        "Set the GUI filelist"
        self.filecombo.clear()

        if self.recentFiles:
            self.filecombo.addItems([os.path.split(file)[1] for file in self.recentFiles])
        else:
            self.filecombo.addItem("(none)")
              
    def resendData(self):
        combotext = unicode(self.filecombo.currentText())
        if combotext == "(none)":
            QMessageBox.information( None, "Error saving data", "Unable to send data. Select first a file name by clicking the '...' button.", QMessageBox.Ok + QMessageBox.Default)
            return
        filename=self.recentFiles[self.filecombo.currentIndex()]
        filenameWithoutExt=lower(os.path.splitext(filename)[0])
        self.runNMF()
        self.send("W", Orange.data.Table(filenameWithoutExt + "_W"))
        self.send("H", Orange.data.Table(filenameWithoutExt + "_H"))
        self.send("S", Orange.data.Table(filenameWithoutExt + "_S"))
        self.send("WSH", Orange.data.Table(filenameWithoutExt + "_WSH"))
        self.send("Residuals", Orange.data.Table(filenameWithoutExt + "_Residuals"))
        self.send("qqplot", Orange.data.Table(filenameWithoutExt + "_QQplot"))       
        self.send("Reordered X", Orange.data.Table(filenameWithoutExt + "_Reordered_X"))
        self.send("Reordered WSH", Orange.data.Table(filenameWithoutExt + "_Reordered_WSH"))
        self.send("Reordered residuals", Orange.data.Table(filenameWithoutExt + "_Reordered_residuals"))


    def commit(self):
        self.runNMF()
        self.send("W", self.WW)
        self.send("H", self.HH)
        self.send("S",self.S)
        self.send("WSH",self.WSH)
        self.send("Residuals",self.residuals)
        self.calculateQQPlotValues()
        self.send("qqplot",self.qqplot)
        self.reorder()        
        self.send("Reordered X", self.Xordered)
        self.send("Reordered WSH", self.WSHordered)
        self.send("Reordered residuals", self.residualsOrdered)
        if self.saveOutputs:
            self.saveFile()
        
        
        
            
# Test the widget, run from prompt

if __name__=="__main__":
    appl = QApplication(sys.argv)
    ow = OWNMF()
    ow.show()
    dataset = orange.ExampleTable('C:/Users/Fajwel/Dropbox/Orange/data.tab')
    ow.dataX(dataset)
    appl.exec_()





"""
@author: Fajwel
"""

# Perform Robust SVD

"""        
M0 is the running matrix (to be factorized);
M0 <- M0-(model fit) before calculating next component 
	(M0 is restored before returning to calling module)
M is the running matrix within ALS/LTS loop.
Outliers are treated as missing values
"""
import numpy as np

class rSVD():
    def __init__(self):
        self.aa=0
        
    def isMissing(self,X):
        return X==-99999999
        
    def rank(self,x):
        # same as argsort but does not take into account missing values
        isMis=self.isMissing(x)
        isNotMis=np.logical_not(isMis)
        nbMis=sum(isMis)        
        x[isMis]=max(x[isNotMis])
        xSorted=np.argsort(x)        
        return xSorted[0:(len(x)-nbMis)]

    def doRSVD(self,M0,nTrial,MaxIter,StdGlobLoc,LTScov,ncUser,StepIter,MaxNonImpr,tolerance):       
        missVal=-99999999 # declare missing values as equal to -99999999        
        n=M0.shape[0]
        p=M0.shape[1]
        #MtPct=missVal 
        #MwPct=missVal
        if StdGlobLoc>=2:
            npcov=np.round(n*p*LTScov);
            ncov=np.round(n*LTScov);
            pcov=np.round(p*LTScov);
            nptrim=n*p-npcov
        else:
            nptrim=0
        
        if not(nptrim):
            StdGlobLoc=1
        nc=min(n,p,ncUser)
        #Mmsr=np.zeros(nc);
        Mev=np.zeros(nc);
        if StdGlobLoc>1:
            MaxTrial=nTrial
        else:
            MaxTrial=1

        M0init=M0.copy()

        Mw=np.zeros([p,nc])
        Mt=np.zeros([n,nc])
        #MtPct=missVal
        #MwPct=missVal
        w=np.zeros(p)
        t=np.zeros(n)

        if not(np.any(self.isMissing(M0))) and StdGlobLoc==1:
            FastCode=1
        else:
            FastCode=0

        j1=0
        j2=p
        
        self.t=[]
        #On sauve les missing valeurs de depart (utile pour la suite)
        MMis0=self.isMissing(M0)
        
        for k in range(0,nc):
            for iTrial in range(1, MaxTrial+1):
                #Status1=Status0||"Ncomp "||Char(k)||" Trial "||Char(iTrial)||": ";
                M=M0.copy()
                if StdGlobLoc>1:
                    #Select a random subset
                    M=M.reshape(n*p)
                    M[np.random.permutation(n*p)[npcov:n*p]]=missVal
                    M=M.reshape((n,p))
                MMis=self.isMissing(M)
    
                # Initialize w
                for j in range(0,p):
                    w[j]=np.median(M[:,j][M[:,j]!=missVal]) 
                
                if not(sum(w>0)):
                    w=np.ones(p) 
                w=w/np.sqrt(np.dot(w,w))

                # initialize t (LTS stochastic)
                if not(FastCode):
                    for i in range(0,n):
                        cells = np.logical_not(MMis[i,:])
                        if sum(cells)>1:
                            t[i] = np.dot(M[i,cells],w[cells])/np.dot(w[cells],w[cells])
                        else:
                            t[i] = missVal # Pas assez de valeurs non missing dans la ligne i
                else:
                    t = np.dot(M,w)/np.dot(w,w)
    
                self.t.append(t)                
                t[self.isMissing(t)]=np.median(t[t!=missVal])
                
    
                if StdGlobLoc>=2:
                    Mdiff=abs(M0-np.dot(np.array([t]).T,np.array([w])))
                    Mdiff[MMis0]=missVal
                    # Exclude outlier cells on a global basis
                    RMdiff=self.rank(np.reshape(Mdiff,n*p))
                    cells=RMdiff[0:min(npcov,len(RMdiff))]
                    M=missVal*np.ones(n*p)
                    M[cells]=M0.reshape(n*p)[cells]
                    M=M.reshape((n,p))
                    MMis=self.isMissing(M)
    
    
                iteration=0
                cont=1
                
                while (cont and iteration<MaxIter):
                    # build w
                    if not(FastCode):
                        for j in range(0,p):
                            cells=np.logical_not(MMis[:,j])
                            if (iteration>=0 and StdGlobLoc==3): #Add constraint on col coverage
                                if sum(cells)<ncov :
                                    RMdiff=self.rank(Mdiff[:,j]);
                                    cells=RMdiff[0:min(ncov,len(RMdiff))]
                                    M[cells,j]=M0[cells,j];
                            if sum(cells)>1:
                                w[j] = np.dot(t[cells],M[cells,j])/np.dot(t[cells],t[cells])
                            else:
                                w[j] = missVal
                    else:
                        w = np.dot(M.T,t)/np.dot(t,t)
                        
                    w[self.isMissing(w)]=np.median(w[w!=missVal])
                    w=w/np.sqrt(np.dot(w,w)) #normalize w
                    if StdGlobLoc>=2:
                        Mdiff=abs(M0-np.dot(np.array([t]).T,np.array([w])))
                        Mdiff[MMis0]=missVal
                        # Exclude outlier cells on a global basis
                        RMdiff=self.rank(np.reshape(Mdiff,n*p))
                        cells=RMdiff[0:min(npcov,len(RMdiff))]
                        M=missVal*np.ones(n*p)
                        M[cells]=M0.reshape(n*p)[cells]
                        M=M.reshape((n,p))                        
                        MMis=self.isMissing(M)            
                	 # build t
                    if not(FastCode):
                        for i in range(0,n):
                            cells=np.logical_not(MMis)[i,j1:j2]
                            if (iteration>=0 and StdGlobLoc==3): #Add constraint on row coverage
                                if sum(cells)<pcov:
                                    RMdiff=self.rank(Mdiff[i,j1:j2])
                                    cells=RMdiff[0:min(pcov,len(RMdiff))]
                                    M[i,cells]=M0[i,cells]
                            if sum(cells)>1:
                                t[i] = np.dot(M[i,cells],w[cells])/np.dot(w[cells],w[cells])
                            else:
                                t[i] = missVal
                    else:
                        t = np.dot(M,w)/np.dot(w,w)
                
                    t[self.isMissing(t)]=np.median(t[t!=missVal])
                
                    # note: only w is normalized within loop, t is normalized after convergence
                    
                    if StdGlobLoc>=2:
                        Mdiff=abs(M0-np.dot(np.array([t]).T,np.array([w])))
                        Mdiff[MMis0]=missVal
                        # Exclude outlier cells on a global basis
                        RMdiff=self.rank(np.reshape(Mdiff,n*p))
                        cells=RMdiff[0:min(npcov,len(RMdiff))]
                        M=missVal*np.ones(n*p)
                        M[cells]=M0.reshape(n*p)[cells]
                        M=M.reshape((n,p))
                        MMis=self.isMissing(M)          
                        
                    if np.mod(iteration,StepIter)==0:
                        if StdGlobLoc==1:
                            Mdiff=abs(M0-np.dot(np.array([t]).T,np.array([w])))
                            Mdiff[MMis0]=missVal
                            # Status=Status1||char(round(iter*100/MaxIter,0))||"%";
                            # wait(0);
                            # here:MyStatusMsg(Status);
                        cells=np.logical_not(MMis)
                        Vdiff=Mdiff[cells];
                        diff=np.dot(Vdiff,Vdiff)/sum(sum(cells))
                        """
                    		if LogIter:
                                if StdGlobLoc>=2:
                                    write("Ncomp: ",k," Trial: ",itrial," Iter: ",iter," MSR: ",diff[1]," Elapsed time: ",char(today()-start),"\!N")
                                    ,
                                    write("Ncomp: ",k," Iter: ",iter," MSR: ",diff[1]," Elapsed time: ",char(today()-start),"\!N")
                                    );
                                    );
                        """
                        if iteration==0:
                            BackDiff=diff
                            BackMw=w
                            BackMt=t
                            NonImpr=0
                        elif abs(diff-diff0)/diff0 < tolerance:
                            cont=0 #Permet de sortir de la boucle
                        elif diff>BackDiff:
                            NonImpr+=1
                            if (NonImpr>MaxNonImpr and MaxNonImpr):
                                cont=0
                        elif diff<=BackDiff:
                            BackDiff=diff
                            BackMw=w
                            BackMt=t
                            NonImpr=0
                        diff0=diff
                    iteration+=1
        
                diff=BackDiff
                w=BackMw
                t=BackMt
                #save trial
                if iTrial==1 or diff<DiffTrial:
                    #BestTrial=iTrial
                    DiffTrial=diff
                    tTrial=t.copy()
                    wTrial=w.copy()
                
                # if (LogTrials,write("Ncomp: ",k," Trial: ",iTrial," MSR: ",diff[1],"\!N"));


            # if (LogTrials,write("Ncomp: ",k," Best trial: ",BestTrial," MSR: ",DiffTrial[1],"\!N"))

            t=tTrial
            w=wTrial

            Mw[:,k]=w
            # compute eigen value
            if StdGlobLoc>1:
                # Robust regression of M on tw`
                t=t/np.sqrt(np.dot(t,t)) # Normalize t
                Mt[:,k]=t
                RMdiff=self.rank(np.reshape(Mdiff,n*p)) 
                Ycells=M0.reshape(n*p)[RMdiff][0:min(npcov,len(RMdiff))]
                Xcells=np.reshape(np.dot(np.array([t]).T,np.array([w])),n*p)[RMdiff][0:min(npcov,len(RMdiff))]
                Mev[k] = np.dot(Ycells,Xcells)/np.dot(Xcells,Xcells)
            else:
                Mev[k]=np.sqrt(np.dot(t,t))
                Mt[:,k]=t/Mev[k] # normalize t
    
            """
            if k==1:
                    Mmsr[k]=Mev[k]/trace0
                    ,
                    Mmsr[k]=Mmsr[k-1]+Mev[k]/trace0
                    );
            """
            #remove t*w'
            M0=M0-Mev[k]*np.dot(np.array([Mt[:,k]]).T,np.array([Mw[:,k]])) 
            M0[MMis0]=missVal

        # Mmsr=1-Mmsr;
     
        if nc>1:
            RMev=np.argsort(-Mev)
            Mev=Mev[RMev]
            Mw0=Mw.copy()
            Mt0=Mt.copy()
            for k in range(0,nc):
                Mw[:,k]=Mw0[:,RMev[k]]
                Mt[:,k]=Mt0[:,RMev[k]]

        M0=M0init # Restore M0
        self.U=Mt
        self.V=Mw
        self.D=Mev
        





"""
<name>rSVD</name>
<description>Robust Singular Value Decomposition</description>
<icon>icons/NMF.png</icon>
<priority>30</priority>
"""


import numpy as np
import scipy.stats

#import sys
#sys.path.append("C:\\Python27\\Lib\site-packages\\Orange\\OrangeWidgets")
#sys.path.append("C:\\Users\\Fajwel\\Dropbox\\Orange\\rSVD")
import rSVD
from OWWidget import *
import OWGUI
import re, os.path
from exceptions import Exception


class OWrSVD(OWWidget):
    settingsList = ['rank','maxIter','nTrial','method','saveOutputs',"recentFiles","selectedFileName"]
    # things for saving:    
    savers = {".txt": orange.saveTxt, ".tab": orange.saveTabDelimited,
              ".names": orange.saveC45, ".test": orange.saveC45, ".data": orange.saveC45,
              ".csv": orange.saveCsv
              }

    # exclude C50 since it has the same extension and we do not need saving to it anyway
    registeredFileTypes = [ft for ft in orange.getRegisteredFileTypes() if len(ft)>3 and ft[3] and not ft[0]=="C50"]

    dlgFormats = 'Tab-delimited files (*.tab)\nHeaderless tab-delimited (*.txt)\nComma separated (*.csv)\nC4.5 files (*.data)\nRetis files (*.rda *.rdo)\n' \
                 + "\n".join("%s (%s)" % (ft[:2]) for ft in registeredFileTypes) \
                 + "\nAll files(*.*)"

    savers.update(dict((lower(ft[1][1:]), ft[3]) for ft in registeredFileTypes))
    re_filterExtension = re.compile(r"\(\*(?P<ext>\.[^ )]+)")
    
    def __init__(self, parent=None, signalManager=None):
        OWWidget.__init__(self, parent, signalManager, 'rSVD')
        
        self.inputs = [("Data", ExampleTable, self.dataX),("Xd", ExampleTable, self.dataXd),("Xa", ExampleTable, self.dataXa)]
        self.outputs = [("U", ExampleTable), ("V", ExampleTable),("D", ExampleTable),("UDV",ExampleTable),("Residuals",ExampleTable),("qqplot",ExampleTable)]
        
        self.rank=5
        self.nTrial=10
        self.maxIter=200
        self.method=0
        self.saveOutputs=1
        
        # things for saving
        self.recentFiles=[]
        self.selectedFileName = "None"
        self.data = None
        self.filename = ""  
        
        self.loadSettings()

        # GUI
        box = OWGUI.widgetBox(self.controlArea, "Info")
        self.infoa = OWGUI.widgetLabel(box, 'No data on input yet, waiting to get something.')
        self.infob = OWGUI.widgetLabel(box, '')

        OWGUI.separator(self.controlArea)
        self.paramBox = OWGUI.widgetBox(self.controlArea, "Parameters")
        OWGUI.spin(self.paramBox, self, 'rank', min=1, max=200, step=1,
                   label='Rank',labelWidth=150)
        OWGUI.lineEdit(self.paramBox, self, 'maxIter', label='Maximum number of iterations',labelWidth=150, orientation="horizontal")
        OWGUI.lineEdit(self.paramBox, self, 'nTrial', label='Number of trials',labelWidth=150, orientation="horizontal")
        self.methodBox = OWGUI.widgetBox(self.controlArea, "Method")        
        OWGUI.radioButtonsInBox(self.methodBox, self, 'method', callback=self.setNbIter,
              btnLabels = ["SVD", "rSVD LTS Global", "rSVD LTS Global Restricted"],
              tooltips = ["Singular Value Decomposition",
                          "Robust Singular Value Decomposition (LTS Global)",
                          "Robust Singular Value Decomposition (LTS Global Restricted)"])
    
        OWGUI.button(self.controlArea, self, "Commit", callback=self.commit)
        self.saveBox = OWGUI.widgetBox(self.controlArea, "Saving options")
        OWGUI.checkBox(self.saveBox, self, "saveOutputs", "Save outputs")
        rfbox = OWGUI.widgetBox(self.saveBox, "Filename", orientation="horizontal", addSpace=True)
        self.filecombo = OWGUI.comboBox(rfbox, self, "filename")
        self.filecombo.setMinimumWidth(200)
        button = OWGUI.button(rfbox, self, '...', callback = self.browseFile, disabled=0)
        button.setIcon(self.style().standardIcon(QStyle.SP_DirOpenIcon))
        button.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Fixed)
        self.resend = OWGUI.button(self.saveBox, self, "Retreive data", callback = self.resendData, default=True)
        
        OWGUI.rubber(self.controlArea)
        self.setFilelist()
        #self.resize(260,100)
        self.filecombo.setCurrentIndex(0)

        if self.selectedFileName != "":
            if os.path.exists(self.selectedFileName):
                self.openFile(self.selectedFileName)
            else:
                self.selectedFileName = ""      
        
        self.infob = OWGUI.widgetLabel(box, '')        
        self.paramBox.setDisabled(1)
        self.methodBox.setDisabled(1)
        self.saveBox.setDisabled(1)
        
        self.resize(100,50)        
        
    def dataX(self, dataset):
        if dataset:
            self.dataset = dataset
            self.infoa.setText('%d variables in input data set' % len(dataset[0]))
            self.infob.setText('%d observations in input data set' % len(dataset))
            self.rank = min(len(self.dataset),len(self.dataset[0]))            
            self.paramBox.setDisabled(0)
            self.methodBox.setDisabled(0)
            self.saveBox.setDisabled(0)
        else:
            self.send("U", None)
            self.send("D", None)
            self.send("UDV",None)
            self.send("V", None)
            self.send("Residuals",None)
            self.send("qqplot",None)
            self.infoa.setText('No data on input yet, waiting to get something.')
            self.infob.setText('')
            

    def dataXd(self, dataset):
        if not dataset:
            self.Xd=None
        else:
            self.Xd = dataset   
    
    def dataXa(self, dataset):
        if not dataset:
            self.Xa=None
        else:
            self.Xa= dataset 
    
    def setNbIter(self):
        if self.method==0:
            self.maxIter=200
        elif self.method==1:
            self.maxIter=200
        else:
            self.maxIter=200

    def runSVD(self):
        # treat missing values as equal to -99999999
        M0=np.ones([len(self.dataset),len(self.dataset[0])])
        for i in range(0,len(self.dataset)):
            for j in range(0,len(self.dataset[0])): 
                if self.dataset[i][j].isSpecial():
                    M0[i][j]=-99999999
                else:
                    M0[i][j]=self.dataset[i][j]
#        M0=self.dataset.toNumpy()
#        M0=np.array(M0[0])
        StdGlobLoc=self.method + 1 
        model= rSVD.rSVD()
        nTrial=self.nTrial
        MaxIter=self.maxIter
        LTScov=0.95
        ncUser=self.rank
        StepIter=1
        MaxNonImpr=5
        tolerance=10**(-6)
        model.doRSVD(M0,nTrial,MaxIter,StdGlobLoc,LTScov,ncUser,StepIter,MaxNonImpr,tolerance)
        U=model.U
        D=model.D
        V=model.V
        UDV=np.dot(np.dot(U,np.diag(D)),V.T)       
        residuals=M0-UDV
        residuals[M0==-99999999]=-99999999 #missing values
        self.residualsNP=residuals
        domainX=self.dataset.domain
     # Transform to Orange format   
        self.V=Orange.data.Table(V)        
        # concatenate with Xa if available
        if hasattr(self, 'Xa') and self.Xa:
            self.V=Orange.data.Table([self.V,self.Xa])
        else:
            self.V.domain.add_meta(Orange.feature.Descriptor.new_meta_id(), Orange.feature.String("Variable"))
            # Get the names of the variables as meta attribute
            for i in range(0,len(domainX)):
                self.V[i]["Variable"]=domainX[i].name
        
        newDomain=domainX
        domW=[]
        for i in range(0, self.rank):
            domW.append(Orange.feature.Continuous("Component %i" %(i+1)))
        domW=Orange.data.Domain(domW,0)
        domW.addmetas(domainX.getmetas())
        self.U = Orange.data.Table(domW,U)
        self.UDV = Orange.data.Table(newDomain,UDV)
        self.D=np.array([np.arange(1,len(D)+1),D])
        self.D=self.D.transpose()
        domD=Orange.data.Domain([Orange.feature.Continuous("Number"),Orange.feature.Continuous("Scaling factor")],0)
        self.D=Orange.data.Table(domD,self.D)
        self.residuals= Orange.data.Table(newDomain,np.array(residuals))
        # add initial missing values
        for i in range(0, len(self.residuals)):
            for j in range(0, len(self.residuals[0])):
                if self.residualsNP[i,j]==-99999999:
                    self.residuals[i][j]='?'
            
                           
        if hasattr(self, 'Xd') and self.Xd:
            self.U =  Orange.data.Table([self.U,self.Xd]) 
            self.UDV = Orange.data.Table([self.UDV,self.Xd])
            self.residuals = Orange.data.Table([self.residuals,self.Xd])
        else:
            for i in range(0,len(self.dataset)):
                self.U[i]["ID"]=self.dataset[i]["ID"]
                self.UDV[i]["ID"]=self.dataset[i]["ID"]
                self.residuals[i]["ID"]=self.dataset[i]["ID"]
                
            
    def calculateQQPlotValues(self):
        nbValues=self.residualsNP.shape[0]*self.residualsNP.shape[1]
        # put all residuals in one vector        
        residualsVector=np.reshape(self.residualsNP, nbValues)
        residualsVector=residualsVector[residualsVector!=-99999999] # exclude missing values
        # sort residuals
        residualsVector.sort()
        normalQuantilesVector=scipy.stats.norm.ppf(np.linspace(1./nbValues,1-1./nbValues,nbValues),loc=residualsVector.mean(),scale=residualsVector.std())
        self.qqplot=np.array([normalQuantilesVector,residualsVector])
        self.qqplot=self.qqplot.transpose()
        domQQPlot=Orange.data.Domain([Orange.feature.Continuous("normal quantiles"),Orange.feature.Continuous("residuals quantiles")],0)
        self.qqplot=Orange.data.Table(domQQPlot,self.qqplot)
        
    
    def browseFile(self):
        if self.recentFiles:
            startfile = self.recentFiles[0]
        else:
            startfile = os.path.expanduser("~/")

#        filename, selectedFilter = QFileDialog.getSaveFileNameAndFilter(self, 'Save Orange Data File', startfile,
#                        self.dlgFormats, self.dlgFormats.splitlines()[0])
#        filename = str(filename)
#       The preceding lines should work as per API, but do not; it's probably a PyQt bug as per March 2010.
#       The following is a workaround.
#       (As a consequence, filter selection is not taken into account when appending a default extension.)
        filename, selectedFilter = QFileDialog.getSaveFileName(self, 'Save Orange Data File', startfile,
                         self.dlgFormats), self.dlgFormats.splitlines()[0]
        filename = unicode(filename)
        if not filename or not os.path.split(filename)[1]:
            return

        ext = lower(os.path.splitext(filename)[1])
        if not ext in self.savers:
            filt_ext = self.re_filterExtension.search(str(selectedFilter)).group("ext")
            if filt_ext == ".*":
                filt_ext = ".tab"
            filename += filt_ext

        self.addFileToList(filename)
        if hasattr(self, 'WW') and self.WW:      
            self.saveFile()

    def saveFile(self, *index):
        self.error()
        #if self.data is not None:
        combotext = unicode(self.filecombo.currentText())
        if combotext == "(none)":
            QMessageBox.information( None, "Error saving data", "Unable to save data. Select first a file name by clicking the '...' button.", QMessageBox.Ok + QMessageBox.Default)
            return
        filename = self.recentFiles[self.filecombo.currentIndex()]
        fileExt = lower(os.path.splitext(filename)[1])
        filenameWithoutExt=lower(os.path.splitext(filename)[0])
        UfileName=filenameWithoutExt + "_U" + ".tab"
        DfileName=filenameWithoutExt + "_D" + ".tab"
        VfileName=filenameWithoutExt + "_V" + ".tab"
        ResidualsfileName=filenameWithoutExt + "_Residuals" + ".tab"
        UDVfileName=filenameWithoutExt + "_UDV" + ".tab"
        QQplotfileName=filenameWithoutExt + "_QQplot" + ".tab"
        
        if fileExt == "":
            fileExt = ".tab"
        try:
            self.savers[fileExt](UfileName, self.U)
            self.savers[fileExt](DfileName, self.D)
            self.savers[fileExt](VfileName, self.V)
            self.savers[fileExt](ResidualsfileName, self.residuals)
            self.savers[fileExt](UDVfileName, self.UDV)
            self.savers[fileExt](QQplotfileName, self.qqplot)
        except Exception, (errValue):
            self.error(str(errValue))
            return
        self.error()

    def addFileToList(self,fn):
        if fn in self.recentFiles:
            self.recentFiles.remove(fn)
        self.recentFiles.insert(0,fn)
        self.setFilelist()

    def setFilelist(self):
        "Set the GUI filelist"
        self.filecombo.clear()

        if self.recentFiles:
            self.filecombo.addItems([os.path.split(file)[1] for file in self.recentFiles])
        else:
            self.filecombo.addItem("(none)")
              
    def resendData(self):
        combotext = unicode(self.filecombo.currentText())
        if combotext == "(none)":
            QMessageBox.information( None, "Error saving data", "Unable to send data. Select first a file name by clicking the '...' button.", QMessageBox.Ok + QMessageBox.Default)
            return
        filename=self.recentFiles[self.filecombo.currentIndex()]
        filenameWithoutExt=lower(os.path.splitext(filename)[0])
        self.runSVD()
        self.send("U", Orange.data.Table(filenameWithoutExt + "_U"))
        self.send("D", Orange.data.Table(filenameWithoutExt + "_D"))
        self.send("V", Orange.data.Table(filenameWithoutExt + "_V"))
        self.send("UDV", Orange.data.Table(filenameWithoutExt + "_UDV"))
        self.send("Residuals", Orange.data.Table(filenameWithoutExt + "_Residuals"))
        self.send("qqplot", Orange.data.Table(filenameWithoutExt + "_QQplot"))

    def commit(self):
        self.runSVD()
        self.send("U", self.U)
        self.send("D", self.D)
        self.send("V",self.V)
        self.send("UDV",self.UDV)
        self.send("Residuals",self.residuals)
        self.calculateQQPlotValues()
        self.send("qqplot",self.qqplot)
        if self.saveOutputs:
            self.saveFile()
        
        
        
            
# Test the widget, run from prompt

if __name__=="__main__":
    appl = QApplication(sys.argv)
    ow = OWrSVD()
    ow.show()
    dataset = orange.ExampleTable('C:/Users/Fajwel/Dropbox/Orange/data.tab')
    ow.dataX(dataset)
    appl.exec_()







"""
<name>Selection for NMF</name>
<description>Selection of variables and observations</description>
<icon>icons/NMF.png</icon>
<priority>30</priority>
"""

#import sys
#sys.path.append("C:\\Python27\\Lib\site-packages\\Orange\\OrangeWidgets")
from OWWidget import *
import OWGUI



class OWSelectDataNMF(OWWidget):
    settingsList = []
    def __init__(self, parent=None, signalManager=None):
        OWWidget.__init__(self, parent, signalManager, 'Selection for NMF')
        # X corresponds to numeric attributes (X will be factorized in X=W*H)
        
        self.inputs = [("X", ExampleTable, self.data),("Xd", ExampleTable, self.dataXd),("Xa", ExampleTable, self.dataXa),("List of variables", ExampleTable, self.dataVar),("List of observations", ExampleTable, self.dataObs)]
        self.outputs = [("Selected data", ExampleTable),("Selected Xd", ExampleTable),("Selected Xa", ExampleTable)]
        self.loadSettings()

        # GUI
        box = OWGUI.widgetBox(self.controlArea, "Info")
        self.infoa = OWGUI.widgetLabel(box, 'No data on input yet, waiting to get something.')
        self.infob = OWGUI.widgetLabel(box, '')

        OWGUI.separator(self.controlArea)
        OWGUI.button(self.controlArea, self, "Commit", callback=self.commit)
      
        
        self.resize(100,50)

    def data(self, dataset):
        if dataset:
            self.dataset = dataset
            self.infoa.setText('%d variables in input data set' % len(dataset[0]))
            self.infob.setText('%d observations in input data set' % len(dataset))
            self.commit()
        else:
            self.send("Selected data", None)
            self.send("Selected Xd", None)
            self.send("Selected Xa", None)
            self.infoa.setText('No data on input yet, waiting to get something.')
            self.infob.setText('')
            
    def dataXd(self, dataset):
        if not dataset:
            self.Xd=None
        else:
            self.Xd = dataset   
    
    def dataXa(self, dataset):
        if not dataset:
            self.Xa=None
        else:
            self.Xa= dataset 
    
    def dataVar(self, dataset):
        if not dataset:
            self.varList=None
        else:
            self.varList = dataset

    def dataObs(self, dataset):
        if not dataset:
            self.obsList=None
        else:
            self.obsList = dataset        
        
    def readData(self,dataset): # convert from Orange format to Numpy format, keepy only the matrix to factorize
        V=dataset.toNumpy()
        V=V[0]
        return V                
        
    def selectData(self):
        domainX=self.dataset.domain    
        self.selectedData=self.dataset
        if hasattr(self, 'Xa') and self.Xa:        
            self.selectedXa=self.Xa
        if hasattr(self, 'Xd') and self.Xd:
            self.selectedXd=self.Xd           
        if hasattr(self, 'varList') and self.varList:
            # create new domains for X containing the names of the variables in varList (including meta-attributes)
            newDomainListX=[]
            for i in range(0,len(self.varList)):
                if domainX.__contains__(self.varList[i][0].value) and not(domainX.has_meta(self.varList[i][0].value)): #add attribute which is not meta
                    newDomainListX.append(self.varList[i][0].value)
            # return new orange data tables from the new domains
            newDomainX=Orange.data.Domain(newDomainListX,0,domainX)
            # begin by adding all meta-attributes and then remove those which are not in the list of selected variables (except ID)            
            newDomainX.addmetas(domainX.getmetas())
            metas=domainX.getmetas().values()
            varList2=[] # varList2 is the varList but not in Orange format
            for i in range(0, len(self.varList)):
                varList2.append(self.varList[i][0].value)
            for i in range(0,len(metas)):
                if not(varList2.__contains__(metas[i].name)) and not(metas[i].name=="ID"): # this variable is not selected and is meta attribute which is not "ID"
                    newDomainX.removemeta(metas[i])
            self.selectedData=Orange.data.Table(newDomainX,self.dataset)
            if hasattr(self, 'Xa') and self.Xa:
                 # create list of variables            
                varListAll=[]
                for i in range(0, len(self.Xa)):
                    varListAll.append(self.Xa[i]["Variable"].value)
                # create binary array of indices to select in self.Xa
                indSelectionXa=[]
                for i in range(0, len(varListAll)):
                    # check if varListAll[i] is in varList2
                    j=0
                    check=False
                    while not check and j<len(varList2):
                        if varListAll[i]==varList2[j]:
                            indSelectionXa.append(True)
                            check=True
                        j+=1
                    if not check:
                        indSelectionXa.append(False)
                self.indSelectionXa=indSelectionXa
                self.selectedXa=self.selectedXa.select(indSelectionXa)
                
        if hasattr(self, 'obsList') and self.obsList:
            # create list of IDs            
            IDs=[]
            for i in range(0, len(self.dataset)):
                IDs.append(self.dataset[i]["ID"].value)
            # create list of IDs to select
            IDselection=[]
            for i in range(0, len(self.obsList)):
                IDselection.append(self.obsList[i][0].value)
            # create binary array of indices to select in self.dataset
            indSelection=[]
            for i in range(0, len(IDs)):
                # check if IDs[i] is in IDselection
                j=0
                check=False
                while not check and j<len(IDselection):
                    if IDs[i]==IDselection[j]:
                        indSelection.append(True)
                        check=True
                    j+=1
                if not check:
                    indSelection.append(False)
            self.indSelection=indSelection
            self.selectedData=self.selectedData.select(indSelection)
            if hasattr(self, 'Xa') and self.Xa:
                self.selectedXd=self.selectedXd.select(indSelection)
    
    def commit(self):
        self.selectData()    
        self.send("Selected data", self.selectedData)
        if hasattr(self, 'Xd') and self.Xd:
            self.send("Selected Xd", self.selectedXd)
        if hasattr(self, 'Xa') and self.Xa:
            self.send("Selected Xa", self.selectedXa)

            
# Test the widget, run from prompt

if __name__=="__main__":
    appl = QApplication(sys.argv)
    ow = OWSelectDataNMF()
    ow.show()
    dataset = orange.ExampleTable('C:/Users/Fajwel/Dropbox/Orange/testSelectData.tab')
    selectVar=orange.ExampleTable('C:/Users/Fajwel/Dropbox/Orange/testSelectDataVar.tab')
    selectID=orange.ExampleTable('C:/Users/Fajwel/Dropbox/Orange/testSelectDataID.tab')
    ow.data(dataset)
    ow.dataVar(selectVar)
    ow.dataObs(selectID)
    ow.selectData()
    appl.exec_()




"""
<name>Missing Values Indicator</name>
<description>Detection of Missing Values</description>
<icon>icons/NMF.png</icon>
<priority>30</priority>
"""

import numpy as np

#import sys
#sys.path.append("C:\\Python27\\Lib\site-packages\\Orange\\OrangeWidgets")
from OWWidget import *
import OWGUI



class OWMissingValues(OWWidget):
    settingsList = []
    def __init__(self, parent=None, signalManager=None):
        OWWidget.__init__(self, parent, signalManager, 'Missing Values Indicator')
        
        self.inputs = [("Data", ExampleTable, self.data)]
        self.outputs = [("Missing Values Indicator", ExampleTable)]
        self.loadSettings()

        # GUI
        box = OWGUI.widgetBox(self.controlArea, "Info")
        self.infoa = OWGUI.widgetLabel(box, 'No data on input yet, waiting to get something.')
        self.infob = OWGUI.widgetLabel(box, '')

        OWGUI.separator(self.controlArea)
        OWGUI.button(self.controlArea, self, "Commit", callback=self.commit)        
        self.resize(100,50)

    def data(self, dataset):
        if dataset:
            self.dataset = dataset
            self.infoa.setText('%d variables in input data set' % len(dataset[0]))
            self.infob.setText('%d observations in input data set' % len(dataset))
        else:
            self.send("Missing Values Indicator", None)
            self.infoa.setText('No data on input yet, waiting to get something.')
            self.infob.setText('')
                      
        
    def commit(self):
        # find missing value and report them in indicator matrix
        indic=np.ones([len(self.dataset),len(self.dataset[0])]) #last column of data is the IDs
        for i in range(0,len(self.dataset)):
            for j in range(0,len(self.dataset[0])): 
                if self.dataset[i][j].isSpecial():
                    indic[i][j]=0
        self.outputData=indic
        domainX=self.dataset.domain
        self.outputData=Orange.data.Table(domainX, np.array(self.outputData))       
        for i in range(0,len(self.dataset)):        
            self.outputData[i]["ID"]=self.dataset[i]["ID"]
        self.send("Missing Values Indicator", self.outputData)     

            
# Test the widget, run from prompt

if __name__=="__main__":
    appl = QApplication(sys.argv)
    ow = OWMissingValues()
    ow.show()
    dataset = orange.ExampleTable('C:/Users/Fajwel/Dropbox/Orange/testMissingData.txt')
    ow.data(dataset)
    appl.exec_()




"""
<name>Normalization for NMF</name>
<description>Normalization for NMF</description>
<icon>icons/NMF.png</icon>
<priority>30</priority>
"""

import numpy as np

#import sys
#sys.path.append("C:\\Python27\\Lib\site-packages\\Orange\\OrangeWidgets")
from OWWidget import *
import OWGUI



class OWNormalizationNMF(OWWidget):
    settingsList = ['logTransform','scale','scaleMedian','subMin','normRows']
    def __init__(self, parent=None, signalManager=None):
        OWWidget.__init__(self, parent, signalManager, 'Normalization for NMF')
        
        self.inputs = [("Data", ExampleTable, self.data)]
        self.outputs = [("Processed data", ExampleTable)]
        self.logTransform=0
        self.scale=0
        self.scaleMedian=0
        self.subMin=0
        self.normRows=0
        self.loadSettings()

        # GUI
        box = OWGUI.widgetBox(self.controlArea, "Info")
        self.infoa = OWGUI.widgetLabel(box, 'No data on input yet, waiting to get something.')
        self.infob = OWGUI.widgetLabel(box, '')

        OWGUI.separator(self.controlArea)
        self.logBox = OWGUI.widgetBox(self.controlArea, "Log transformation")
        OWGUI.checkBox(self.logBox, self, "logTransform", "Take the log 2 of the data")
        self.normalizeBox = OWGUI.widgetBox(self.controlArea, "Normalization")
        OWGUI.checkBox(self.normalizeBox, self, "scale", "Scale")
        OWGUI.checkBox(self.normalizeBox, self, "scaleMedian", "Divide by median")
        OWGUI.checkBox(self.normalizeBox, self, "subMin", "Subtract minimum value")
        OWGUI.checkBox(self.normalizeBox, self, "normRows", "Normalize rows (sum to one)")
        OWGUI.button(self.controlArea, self, "Commit", callback=self.commit)        
        
        self.logBox.setDisabled(1)
        self.normalizeBox.setDisabled(1)        
        self.resize(100,50)

    def data(self, dataset):
        if dataset:
            self.dataset = dataset
            self.infoa.setText('%d variables in input data set' % len(dataset[0]))
            self.infob.setText('%d observations in input data set' % len(dataset))
            self.logBox.setDisabled(0)
            self.normalizeBox.setDisabled(0)
            self.commit()
        else:
            self.send("Processed data", None)
            self.infoa.setText('No data on input yet, waiting to get something.')
            self.infob.setText('')
        
        
    def readData(self,dataset): # convert from Orange format to Numpy format, keepy only the matrix to factorize
        V=dataset.toNumpy()
        V=V[0]
        return V                
        
    def commit(self):
        self.processedData=self.readData(self.dataset)
        self.processedData=self.processedData.astype(float)
        if self.logTransform:
            self.processedData=np.log2(self.processedData)
        if self.scale:
            for k in range(0,self.processedData.shape[1]):
                self.processedData[:,k]=self.processedData[:,k]/sum(abs(self.processedData[:,k]))
        if self.scaleMedian:
            for k in range(0,self.processedData.shape[1]):
                self.processedData[:,k]=self.processedData[:,k]/np.median(self.processedData[:,k])
        if self.subMin:
            for k in range(0,self.processedData.shape[1]):
                self.processedData[:,k]=self.processedData[:,k]-np.min(self.processedData[:,k])
        if self.normRows:
            for k in range(0,self.processedData.shape[0]):
                self.processedData[k,:]=self.processedData[k,:]/sum(self.processedData[k,:])
        domainX=self.dataset.domain
        newDomain=[]
        for i in range(0, len(domainX)):
            newDomain.append(Orange.feature.Continuous(domainX[i].name))
        newDomain=Orange.data.Domain(newDomain,0)
        newDomain.addmetas(domainX.getmetas())
        self.processed=Orange.data.Table(newDomain, self.processedData)
        if len(domainX.getmetas()):
            nameMeta=domainX.getmetas().values()[0].name
            for i in range(0,len(self.dataset)):
                self.processed[i][nameMeta]=self.dataset[i][nameMeta]        
        self.send("Processed data", self.processed)     

            
# Test the widget, run from prompt

if __name__=="__main__":
    appl = QApplication(sys.argv)
    ow = OWNormalizationNMF()
    ow.show()
    dataset = orange.ExampleTable('C:/Users/Paul Fogel/Desktop/testNormRows.tab')
    ow.data(dataset)
    appl.exec_()




"""
<name>Pivot Data</name>
<description>Pivot data</description>
<icon>icons/NMF.png</icon>
<priority>30</priority>
"""

import numpy as np

#import sys
#sys.path.append("C:\\Python27\\Lib\site-packages\\Orange\\OrangeWidgets")
from OWWidget import *
import OWGUI



class OWPivotData(OWWidget):
    settingsList = []
    def __init__(self, parent=None, signalManager=None):
        OWWidget.__init__(self, parent, signalManager, 'Pivot Data')
        
        self.inputs = [("Data", ExampleTable, self.data)]
        self.outputs = [("Pivoted Data", ExampleTable)]
        self.loadSettings()

        # GUI
        box = OWGUI.widgetBox(self.controlArea, "Info")
        self.infoa = OWGUI.widgetLabel(box, 'No data on input yet, waiting to get something.')
        self.infob = OWGUI.widgetLabel(box, '')

        OWGUI.separator(self.controlArea)
        OWGUI.button(self.controlArea, self, "Pivot long", callback=self.pivotLong)
        OWGUI.button(self.controlArea, self, "Pivot short", callback=self.pivotShort)        
        self.resize(100,50)

    def data(self, dataset):
        if dataset:
            self.dataset = dataset
            self.infoa.setText('%d variables in input data set' % len(dataset[0]))
            self.infob.setText('%d observations in input data set' % len(dataset))
        else:
            self.send("Pivoted Data", None)
            self.infoa.setText('No data on input yet, waiting to get something.')
            self.infob.setText('')
                      
        
    def pivotLong(self):
        self.outputData=[]
        colVal=[]
        colNames=[]
        colID=[]
        for j in range(0, len(self.dataset.domain)):
            for i in range(0, len(self.dataset)): 
                colVal.append(self.dataset[i][j].value)
                colNames.append(self.dataset.domain[j].name)
                colID.append(self.dataset[i]["ID"].value)
        self.outputData=map(list, zip(*[colVal]))
        dom=[Orange.feature.Continuous("Value")]
        dom=Orange.data.Domain(dom,0)
        dom.addmetas(self.dataset.domain.getmetas()) #add "ID"
        newid = Orange.feature.Descriptor.new_meta_id()
        dom.add_meta(newid, Orange.feature.String("Names")) #add "Names"
        self.outputData=Orange.data.Table(dom,self.outputData)
        for i in range(0,len(self.outputData)):        
            self.outputData[i]["ID"]=colID[i]
            self.outputData[i]["Names"]=colNames[i]
        self.send("Pivoted Data", self.outputData)

    def pivotShort(self):
        self.outputData=[]
        domList=[]
        nameMeta=self.dataset.domain.getmetas().values()
        if nameMeta[0].name=="ID":
            nameMeta=nameMeta[1].name
        else:
            nameMeta=nameMeta[0].name
        i=0
        while i<len(self.dataset):
            col=[]
            nameCol=self.dataset[i][nameMeta].value
            domList.append(Orange.feature.Continuous(nameCol))
            while (i<len(self.dataset)) and self.dataset[i][nameMeta].value == nameCol:
                col.append(self.dataset[i][0].value)
                i+=1
            self.outputData.append(col)
        self.outputData=map(list, zip(*self.outputData))
        dom=Orange.data.Domain(domList,0)
        dom.addmetas(self.dataset.domain.getmetas()) #add "ID"
        dom.remove_meta(nameMeta)
        self.outputData=Orange.data.Table(dom,self.outputData)
        for i in range(0,len(self.outputData)):        
            self.outputData[i]["ID"]=self.dataset[i]["ID"]
        self.send("Pivoted Data", self.outputData)     

            
# Test the widget, run from prompt

if __name__=="__main__":
    appl = QApplication(sys.argv)
    ow = OWPivotData()
    ow.show()
    dataset = orange.ExampleTable('C:/Users/Paul Fogel/Desktop/testPivot.tab')
    ow.data(dataset)
    appl.exec_()
